000001 IDENTIFICATION  DIVISION.
000002* CALLK32.
000003 PROGRAM-ID.     CALLK32.
000004 ENVIRONMENT     DIVISION.
000005 CONFIGURATION   SECTION.
000010 SPECIAL-NAMES.
000011 REPOSITORY.
000012 .
000013 INPUT-OUTPUT    SECTION.
000014 FILE-CONTROL.
000015 DATA            DIVISION.
000020 LINKAGE         SECTION.
000021 01  POW-FORM IS GLOBAL.
000022   02  POW-SELF PIC S9(9) COMP-5.
000023   02  POW-SUPER  PIC X(4).
000024   02  POW-THIS PIC S9(9) COMP-5.
000025 01  CALLK32 REDEFINES POW-FORM GLOBAL PIC S9(9) COMP-5.
000026 01  POW-CONTROL-ID PIC S9(9) COMP-5.
000027 01  POW-EVENT-ID   PIC S9(9) COMP-5.
000028 01  POW-OLE-PARAM  PIC X(4).
000029 01  POW-OLE-RETURN PIC X(4).
000031 PROCEDURE       DIVISION USING POW-FORM POW-CONTROL-ID POW-EVENT-ID POW-OLE-PARAM POW-OLE-RETURN.
000032     EVALUATE POW-CONTROL-ID
000033     WHEN 117440513
000034     EVALUATE POW-EVENT-ID
000035     WHEN 117440768
000036       CALL "POW-SCRIPTLET1"
000037     END-EVALUATE
000038     END-EVALUATE
000039     EXIT PROGRAM.
000041 IDENTIFICATION  DIVISION.
000042* CallK32-Opened.
000043 PROGRAM-ID.     POW-SCRIPTLET1.
000044 ENVIRONMENT DIVISION.
000045 DATA DIVISION.
000046 WORKING-STORAGE SECTION.
000048 *> NOTE: 
000049 *>    The following lines must be included in the COBOL85.CBR file located in the same directory as the
000050 *>    program .EXE file. The 'KER' of [KER.ENTRY] is the name of the form from which the functions are 
000051 *>    called. The '.ENTRY' is a required suffix for this section. If this section is note included the 
000052 *>    program will generate a JMP0015I-U error when run.
000053 *>
000054 *> [CallK32.ENTRY]
000055 *> GetCurrentDirectoryA=KERNEL32
000056 *> GetShortPathNameA=KERNEL32
000057 *> GetLastError=KERNEL32
000059 *>  Define variables to hold KERNEL32 function names.
000060 01  GetCurrentDirectoryA	 PIC X(20) VALUE "GetCurrentDirectoryA".
000061 01  GetShortPathNameA	 PIC X(17) VALUE "GetShortPathNameA".
000062 01  GetLastError  	 PIC X(12) VALUE "GetLastError".
000064 *>  Note the following relationships:
000065 *>  PIC S9(9) COMP-5 corresponds to DWORD or LONG INT
000066 *>  PIC X(???) corresponds to a CHAR string
000067 01  PTR                   PIC 999 VALUE ZERO.         *> general counter variable
000068 01  CALL-RESULT           PIC S9(9) COMP-5.           *> holds function return value
000069 01  LONG-DIRECTORY        PIC X(128).                 *> holds long directory name before conversion
000070 01  LONG-DIRECTORY-LENGTH PIC S9(9) COMP-5 VALUE 128. *> DWORD type. Used to pass length of LONG-DIRECTORY
000071 01  SHORT-DIRECTORY       PIC X(64).                  *> holds short directory name after conversion
000072 01  SHORT-DIR-LENGTH      PIC S9(9) COMP-5 VALUE 64.  *> DWORD type. Used to pass length of SHORT-DIRECTORY
000074 01 POW-0000 PIC S9(18) COMP-5.
000074 01 POW-0001 PIC S9(9) COMP-5.
000074 01 POW-0002 PIC S9(9) COMP-5.
000074 01 POW-0003 PIC S9(9) COMP-5.
000074 01 POW-0004 PIC S9(9) COMP-5.
000074 01 POW-0005 PIC S9(9) COMP-5.
000074 01 POW-0006 PIC X(8192).
000074 01 POW-0007 PIC S9(9) COMP-5.
000074 01 POW-0008 PIC X(8192).
000074 01 POW-0009 PIC S9(9) COMP-5.
000074 01 POW-0010 PIC S9(14)V9(4) COMP-5.
000074 01 POW-0011 PIC S9(9) COMP-5.
000074 01 POW-0012 PIC S9(4) COMP-5.
000074 PROCEDURE DIVISION.
000076     *> Get current directory. Note essential 'WITH STDCALL LINKAGE' phrase, as well as the use of 'BY VALUE' and 'BY REFERENCE'
000077     CALL GetCurrentDirectoryA WITH STDCALL LINKAGE USING  BY VALUE LONG-DIRECTORY-LENGTH BY REFERENCE LONG-DIRECTORY  RETURNING CALL-RESULT.
000078 
000079     *> Check for a too small buffer.
000080     IF CALL-RESULT >= LONG-DIRECTORY-LENGTH
000081        DISPLAY "LONG-DIRECTORY BUFFER IS TOO SMALL"
000082     END-IF.
000083     
000084     *> Check for an error and return extended error code.
000085     IF CALL-RESULT = ZERO
000086        CALL GetLastError RETURNING CALL-RESULT
000087        DISPLAY "CANNOT READ CURRENT DIRECTORY. ERROR: " CALL-RESULT
000088     END-IF.
000090     *> Convert to short path format. Note the use of 'BY CONTENT' phrase.
000091     CALL GetShortPathNameA WITH STDCALL LINKAGE USING BY CONTENT LONG-DIRECTORY BY REFERENCE SHORT-DIRECTORY BY VALUE SHORT-DIR-LENGTH RETURNING CALL-RESULT. 
000093     *> Check for a too small buffer. 
000094     IF CALL-RESULT = SHORT-DIR-LENGTH
000095        DISPLAY "SHORT-DIRECTORY BUFFER IS TOO SMALL"
000096     END-IF.
000097     
000098     *> Check for an error and return extended error code.
000099     IF CALL-RESULT = ZERO
000100        CALL GetLastError RETURNING CALL-RESULT
000101        DISPLAY "CANNOT CONVERT TO SHORT DIRECTORY. ERROR: " CALL-RESULT
000102     END-IF.
000104     *> Remove all LOW-VALUEs
000105     INSPECT SHORT-DIRECTORY REPLACING ALL LOW-VALUE BY SPACE.
000106     PERFORM VARYING PTR FROM SHORT-DIR-LENGTH BY -1
000107        UNTIL SHORT-DIRECTORY(PTR:1) NOT = SPACE
000108        *> Loop and do nothing. This may generate a warning which can be ignored.
000109     END-PERFORM.
000111     *> Add an '\' to the end of the pathname (this routine will handle spaces within directory names).
000112     IF PTR < SHORT-DIR-LENGTH
000113        ADD 1 TO PTR
000114        MOVE "\" TO SHORT-DIRECTORY(PTR:1)
000115     END-IF.
000117     *> Display results so we can see what we have done
000118     MOVE 117441026 TO POW-0000 
000118     MOVE 1 TO POW-0001 
000118     MOVE 16387 TO POW-0003 
000118     MOVE 0 TO POW-0004 
000118     MOVE 3 TO POW-0005 
000118     MOVE LONG-DIRECTORY TO POW-0006 
000118     MOVE 33636360 TO POW-0007 
000118     MOVE "Long Path Name" TO POW-0008 
000118     MOVE 50413576 TO POW-0009 
000118     MOVE 0 TO POW-0010 
000118     MOVE 50413574 TO POW-0011 
000118     CALL "XPOW_INVOKE_BY_ID_2" USING VALUE POW-SELF REFERENCE POW-0000 
000118     VALUE POW-0001 POW-0003 REFERENCE POW-0002 VALUE POW-0004 POW-0005 
000118     POW-0007 REFERENCE POW-0006 VALUE POW-0009 REFERENCE POW-0008 VALUE 
000118     POW-0011 REFERENCE POW-0010 END-CALL 
000118                                                                                      .
000119     MOVE 117441026 TO POW-0000 
000119     MOVE 1 TO POW-0001 
000119     MOVE 16387 TO POW-0003 
000119     MOVE 0 TO POW-0004 
000119     MOVE 3 TO POW-0005 
000119     MOVE SHORT-DIRECTORY TO POW-0006 
000119     MOVE 33636360 TO POW-0007 
000119     MOVE "Short Path Name" TO POW-0008 
000119     MOVE 50413576 TO POW-0009 
000119     MOVE 0 TO POW-0010 
000119     MOVE 50413574 TO POW-0011 
000119     CALL "XPOW_INVOKE_BY_ID_2" USING VALUE POW-SELF REFERENCE POW-0000 
000119     VALUE POW-0001 POW-0003 REFERENCE POW-0002 VALUE POW-0004 POW-0005 
000119     POW-0007 REFERENCE POW-0006 VALUE POW-0009 REFERENCE POW-0008 VALUE 
000119     POW-0011 REFERENCE POW-0010 END-CALL 
000119                                                                                        .
000121     *> End
000122     MOVE 117441038 TO POW-0000 
000122     MOVE 1 TO POW-0001 
000122     MOVE 16395 TO POW-0002 
000122     MOVE 0 TO POW-0003 
000122     MOVE 0 TO POW-0004 
000122     CALL "XPOW_INVOKE_BY_ID_2" USING VALUE POW-SELF REFERENCE POW-0000 
000122     VALUE POW-0001 POW-0002 REFERENCE POW-0012 VALUE POW-0003 POW-0004 
000122     END-CALL 
000122                                 .
000123 END PROGRAM     POW-SCRIPTLET1.
000125 END PROGRAM     CALLK32.
